package com.muyu.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.muyu.common.core.annotation.Excel;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.domain.req.FencesEditReq;
import com.muyu.domain.req.FencesQueryReq;
import com.muyu.domain.req.FencesSaveReq;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.math.BigDecimal;
import java.util.List;


/** 电子围栏
 * @ClassDescription:
 * @JdkVersion: 17
 * @Author: zhangxu
 * @Created: 2024/5/31 15:11
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@SuperBuilder
public class Fences extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /** 围栏id */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "围栏id", value = "围栏id")
    private Long id;

    /** 围栏组id */
    @Excel(name = "围栏组id")
    @ApiModelProperty(name = "围栏组id", value = "围栏组id")
    private Long groupId;

    /** 电子围栏名称 */
    @Excel(name = "电子围栏名称")
    @ApiModelProperty(name = "电子围栏名称", value = "电子围栏名称")
    private String fenceName;

    /** 围栏类型 */
    @Excel(name = "围栏类型")
    @ApiModelProperty(name = "围栏类型", value = "围栏类型")
    private String fenceType;

    /** 半径 */
    @Excel(name = "半径")
    @ApiModelProperty(name = "半径", value = "半径")
    private String radius;

    /** 驶入、驶出 */
    @Excel(name = "驶入、驶出")
    @ApiModelProperty(name = "驶入、驶出", value = "驶入、驶出")
    private String eventType;

    /** 围栏状态 */
    @Excel(name = "围栏状态")
    @ApiModelProperty(name = "围栏状态", value = "围栏状态")
    private String status;

    /** 坐标 */
    @Excel(name = "坐标")
    @ApiModelProperty(name = "坐标", value = "坐标")
    private String polygonPoints;

    /**
     * 围栏坐标
     */
    @Excel(name = "坐标")
    @ApiModelProperty(name = "坐标", value = "坐标")
    @TableField(exist = false)
    private List<Path> path;

    /**
     * 查询构造器
     */
    public static Fences queryBuild( FencesQueryReq fencesQueryReq){
        return Fences.builder()
                .groupId(fencesQueryReq.getGroupId())
                .fenceName(fencesQueryReq.getFenceName())
                .fenceType(fencesQueryReq.getFenceType())
                .radius(fencesQueryReq.getRadius())
                .eventType(fencesQueryReq.getEventType())
                .status(fencesQueryReq.getStatus())
                .polygonPoints(fencesQueryReq.getPolygonPoints())
                .build();
    }

    /**
     * 添加构造器
     */
    public static Fences saveBuild(FencesSaveReq fencesSaveReq){
        return Fences.builder()
                .groupId(fencesSaveReq.getGroupId())
                .fenceName(fencesSaveReq.getFenceName())
                .fenceType(fencesSaveReq.getFenceType())
                .radius(fencesSaveReq.getRadius())
                .eventType(fencesSaveReq.getEventType())
                .status(fencesSaveReq.getStatus())
                .polygonPoints(fencesSaveReq.getPolygonPoints())
                .build();
    }

    /**
     * 修改构造器
     */
    public static Fences editBuild(Long id, FencesEditReq fencesEditReq){
        return Fences.builder()
                .id(id)
                .groupId(fencesEditReq.getGroupId())
                .fenceName(fencesEditReq.getFenceName())
                .fenceType(fencesEditReq.getFenceType())
                .radius(fencesEditReq.getRadius())
                .eventType(fencesEditReq.getEventType())
                .status(fencesEditReq.getStatus())
                .polygonPoints(fencesEditReq.getPolygonPoints())
                .build();
    }
}
