package com.muyu.networking.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.domain.FenceGroups;
import com.muyu.domain.req.FenceGroupsEditReq;
import com.muyu.domain.req.FenceGroupsQueryReq;
import com.muyu.domain.req.FenceGroupsSaveReq;
import com.muyu.networking.service.FenceGroupsService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;



/**
 * 电子围栏组Controller
 *
 * @author muyu
 * @date 2024-06-02
 */
@Api(tags = "电子围栏组")
@RestController
@RequestMapping("/groups")
public class FenceGroupsController extends BaseController {
    @Autowired
    private FenceGroupsService fenceGroupsService;

    /**
     * 查询电子围栏组列表
     */
    @ApiOperation("获取电子围栏组列表")
    @GetMapping("/list")
    public Result<TableDataInfo<FenceGroups>> list(FenceGroupsQueryReq fenceGroupsQueryReq) {
        startPage();
        List<FenceGroups> list = fenceGroupsService.list(FenceGroups.queryBuild(fenceGroupsQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出电子围栏组列表
     */
    @ApiOperation("导出电子围栏组列表")
    @Log(title = "电子围栏组", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, FenceGroups fenceGroups) {
        List<FenceGroups> list = fenceGroupsService.list(fenceGroups);
        ExcelUtil<FenceGroups> util = new ExcelUtil<FenceGroups>(FenceGroups.class);
        util.exportExcel(response, list, "电子围栏组数据");
    }

    /**
     * 获取电子围栏组详细信息
     */
    @ApiOperation("获取电子围栏组详细信息")

    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<FenceGroups> getInfo(@PathVariable("id") Long id) {
        return Result.success(fenceGroupsService.getById(id));
    }

    /**
     * 新增电子围栏组
     */

    @Log(title = "电子围栏组", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增电子围栏组")
    public Result<String> add(@RequestBody FenceGroupsSaveReq fenceGroupsSaveReq) {
        return toAjax(fenceGroupsService.save(FenceGroups.saveBuild(fenceGroupsSaveReq)));
    }

    /**
     * 修改电子围栏组
     */

    @Log(title = "电子围栏组", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改电子围栏组")
    public Result<String> edit(@PathVariable Long id, @RequestBody FenceGroupsEditReq fenceGroupsEditReq) {
        return toAjax(fenceGroupsService.updateById(FenceGroups.editBuild(id,fenceGroupsEditReq)));
    }

    /**
     * 删除电子围栏组
     */

    @Log(title = "电子围栏组", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @ApiOperation("删除电子围栏组")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(fenceGroupsService.removeBatchByIds(ids));
    }
}
