package com.muyu.networking.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.domain.Fences;
import com.muyu.domain.req.FencesEditReq;
import com.muyu.domain.req.FencesQueryReq;

import com.muyu.networking.service.FenceService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;


/**
 * 电子围栏Controller
 *
 * @author muyu
 * @date 2024-06-02
 */
@Api(tags = "电子围栏")
@RestController
@RequestMapping("/fences")
public class FencesController extends BaseController {
    @Autowired
    private FenceService fencesService;

    /**
     * 查询电子围栏列表
     */
    @ApiOperation("获取电子围栏列表")
    @RequiresPermissions("system:fences:list")
    @GetMapping("/list")
    public Result<TableDataInfo<Fences>> list(FencesQueryReq fencesQueryReq) {
        startPage();
        List<Fences> list = fencesService.list(Fences.queryBuild(fencesQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出电子围栏列表
     */
    @ApiOperation("导出电子围栏列表")
    @RequiresPermissions("system:fences:export")
    @Log(title = "电子围栏", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Fences fences) {
        List<Fences> list = fencesService.list(fences);
        ExcelUtil<Fences> util = new ExcelUtil<Fences>(Fences.class);
        util.exportExcel(response, list, "电子围栏数据");
    }

    /**
     * 获取电子围栏详细信息
     */
    @ApiOperation("获取电子围栏详细信息")
    @RequiresPermissions("system:fences:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<Fences> getInfo(@PathVariable("id") Long id) {
        return Result.success(fencesService.getById(id));
    }

    /**
     * 新增电子围栏
     */
    @RequiresPermissions("system:fences:add")
    @Log(title = "电子围栏", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增电子围栏")
    public Result<String> add(@RequestBody Fences fences) {
        fencesService.insertFence(fences);
        return  Result.success();
    }

    /**
     * 修改电子围栏
     */
    @RequiresPermissions("system:fences:edit")
    @Log(title = "电子围栏", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改电子围栏")
    public Result<String> edit(@PathVariable Long id, @RequestBody FencesEditReq fencesEditReq) {
        return toAjax(fencesService.updateById(Fences.editBuild(id,fencesEditReq)));
    }

    /**
     * 删除电子围栏
     */
    @RequiresPermissions("system:fences:remove")
    @Log(title = "电子围栏", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @ApiOperation("删除电子围栏")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(fencesService.removeBatchByIds(ids));
    }
}
