//package com.muyu.vehicle.config;
//
//import com.alibaba.druid.pool.DruidDataSource;
//import com.alibaba.druid.pool.DruidPooledConnection;
//import com.alibaba.druid.stat.DruidDataSourceStatManager;
//
//import com.muyu.domain.datasources.Datasource;
//
//import com.muyu.vehicle.context.DataSourceContextHolder;
//import lombok.Data;
//import lombok.extern.log4j.Log4j2;
//import org.springframework.jdbc.datasource.lookup.AbstractRoutingDataSource;
//import java.sql.Connection;
//import java.sql.DriverManager;
//import java.sql.SQLException;
//import java.util.Map;
//import java.util.Objects;
//import java.util.Set;
//
///**
// * @ClassName
// * @Author
// * @Date 2024/4/28 17:01
// */
//@Data
//@Log4j2
//public class DynamicRoutingDataSource extends AbstractRoutingDataSource {
//
//    private boolean debug = true;
//
//    /**
//     * 存储我们注册的数据源
//     */
//    private volatile Map<Object, Object> customDataSources;
//
//    @Override
//    protected Object determineCurrentLookupKey() {
//        Long datasourceId = DataSourceContextHolder.getDataSource();   // 获取当前数据源
//        if(!Objects.isNull(datasourceId)){
//            Map<Object, Object> map = this.customDataSources;   // 存储我们注册的数据源
//            if(map.containsKey(datasourceId)){
//                log.info("当前数据源是：{}",datasourceId);
//            }else{
//                log.info("不存在数据源：{}",datasourceId);
//                return null;
//            }
//        }else{
//            log.info("当前是默认数据源");
//        }
//        return datasourceId;
//    }
//
//    @Override
//    public void setTargetDataSources(Map<Object, Object> param) {
//        super.setTargetDataSources(param);
//        this.customDataSources = param;   // 存储我们注册的数据源
//    }
//
//
//    /**
//     * 检查是否已存在指定ID的数据源，如果存在则验证其连接是否有效，如果无效则重新创建数据源；如果不存在则直接创建数据源。
//     * @param dataSource 需要检查或创建的数据源对象。
//     */
//    public void checkCreateDataSource(Datasource dataSource){
//        // 获取数据源ID
//        Long datasourceId = dataSource.getId();
//        // 获取已注册的数据源映射
//        Map<Object, Object> map = this.customDataSources;
//        // 检查是否已存在对应ID的数据源
//        if(map.containsKey(datasourceId)){
//            // 尝试获取并验证数据源连接
//            // 这里检查一下之前创建的数据源，现在是否连接正常
//            DruidDataSource druidDataSource = (DruidDataSource) map.get(datasourceId);
//            boolean flag = true;
//            DruidPooledConnection connection = null;
//            try {
//                // 尝试获取数据源连接
//                connection = druidDataSource.getConnection();
//            } catch (SQLException throwable) {
//                // 如果获取连接失败，则记录错误信息并标记需要重新创建数据源
//                log.error(throwable.getMessage());
//                flag = false;
//                // 删除已失效的数据源
//                delDataSources(datasourceId);   // 删除数据源
//            }finally {
//                // 确保关闭连接
//                //如果连接正常记得关闭
//                if(null != connection){
//                    try {
//                        connection.close();
//                    } catch (SQLException e) {
//                        log.error(e.getMessage());
//                    }
//                }
//            }
//            // 如果标记为需要重新创建数据源，则进行创建
//            if(!flag){
//                createDataSource(dataSource);   // 创建数据源
//            }
//        }else {
//            // 如果不存在对应ID的数据源，则直接创建
//            createDataSource(dataSource);   // 创建数据源
//        }
//    }
//
//
//    /**
//     * 创建数据源。
//     * 使用Druid数据源工厂类来配置并创建数据源实例。
//     * 此方法主要用于初始化一个特定数据库配置的数据源，并将其添加到自定义数据源的映射中。
//     *
//     * @param dataSource 数据源配置对象，包含数据库连接所需的全部信息。
//     */
//    public void createDataSource(Datasource dataSource) {   // 数据源对象
//        try {
//            // 构建数据库连接URL
//            String url="jdbc:mysql://"+dataSource.getIp();   // 数据库ip
//            String driverType="com.mysql.cj.jdbc.Driver";
//            url+="/"+dataSource.getDatabase()+"?useUnicode=true&characterEncoding=UTF-8&zeroDateTimeBehavior=convertToNull&serverTimezone=UTC&useSSL=false&zeroDateTimeBehavior=CONVERT_TO_NULL&allowPublicKeyRetrieval=true";   // 数据库
//            // 加载MySQL驱动类
//            Class.forName(driverType);
//            // 尝试建立与数据库的连接
//            Connection connection = DriverManager.getConnection(url, dataSource.getUsername(), dataSource.getPassword());   // 用户名 | 密码
//            // 如果连接成功，则关闭连接；如果连接失败，则记录错误日志
//            if(connection==null){
//                log.error("数据源配置有错误，DataSource：{}",dataSource);
//            }else{
//                connection.close();
//            }
//            // 创建DruidDataSource实例用于配置和管理数据库连接
//            DruidDataSource druidDataSource = new DruidDataSource();
//            // 设置数据源名称
//            druidDataSource.setName(dataSource.getId().toString());   // 编号
//            // 设置驱动类名
//            druidDataSource.setDriverClassName(driverType);
//            // 设置数据库连接URL
//            druidDataSource.setUrl(url);
//            // 设置数据库用户名
//            druidDataSource.setUsername(dataSource.getUsername());   // 用户名
//            // 设置数据库密码
//            druidDataSource.setPassword(dataSource.getPassword());   // 密码
//            // 配置连接池的最大活动连接数
//            druidDataSource.setMaxActive(20);
//            // 配置连接池的最小空闲连接数
//            druidDataSource.setMinIdle(5);
//            // 设置连接的最大等待时间
//            // 获取连接最大等待时间，单位毫秒
//            druidDataSource.setMaxWait(6000);
//            // 设置用于检查连接有效性的SQL查询
//            String validationQuery = "select 1";
//            // 开启在获取连接时执行有效性检查的特性
//            // 申请连接时执行validationQuery检测连接是否有效，防止取到的连接不可用
//            druidDataSource.setTestOnBorrow(true);
//            druidDataSource.setValidationQuery(validationQuery);
//            // Druid数据源
//            druidDataSource.init();
//            // 将创建的数据源添加到自定义数据源映射中
//            this.customDataSources.put(dataSource.getId(),druidDataSource);   // 编号
//            // 更新父类的TargetDataSources以包含新的数据源
//            // 将map赋值给父类的TargetDataSources
//            setTargetDataSources(this.customDataSources);   // 存储我们注册的数据源
//            // 标记属性已设置，以触发数据源的初始化
//            // 将TargetDataSources中的连接信息放入resolvedDataSources管理
//            super.afterPropertiesSet();
//
//        } catch (Exception e) {
//            // 记录数据源创建过程中的任何异常
//            log.error("数据源创建失败",e);
//        }
//    }
//
//    /**
//     *  删除数据源
//     * @param datasourceId 数据源编号
//     */
//    private void delDataSources(Long datasourceId) {
//        Map<Object, Object> map = this.customDataSources;   // 存储我们注册的数据源
//        Set<DruidDataSource> druidDataSourceInstances = DruidDataSourceStatManager.getDruidDataSourceInstances();
//        for (DruidDataSource dataSource : druidDataSourceInstances) {
//            if (datasourceId.toString().equals(dataSource.getName())) {
//                map.remove(datasourceId);
//                //从实例中移除当前dataSource
//                DruidDataSourceStatManager.removeDataSource(dataSource);
//                // 将map赋值给父类的TargetDataSources
//                setTargetDataSources(map);
//                // 将TargetDataSources中的连接信息放入resolvedDataSources管理
//                super.afterPropertiesSet();
//            }
//        }
//    }
//
//}
//
