package com.muyu.vehicle.myDatasource;


import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson.JSON;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.SpringUtils;
import com.muyu.common.system.remote.RemoteUserService;
import com.muyu.domain.datasources.EntInfo;
import com.muyu.vehicle.myDatasource.domain.DataSourceInfo;
import com.muyu.vehicle.myDatasource.domain.EnterpriseInfo;
import com.muyu.vehicle.myDatasource.factory.DruidDataSourceFactory;
import com.muyu.vehicle.myDatasource.rule.DynamicDataSource;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassDescription:
 * @JdkVersion: 17
 * @Author: zhangxu
 * @Created: 2024/6/10 20:04
 */
@Log4j2
@Component
public class ManyDataSource {


    @Autowired
    private RemoteUserService remoteEntInfoService;


    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @RabbitListener(queuesToDeclare = {@Queue(name = "RoutingKey")})
    public void smsConfig(String msg, Message message, Channel channel) {
        //获取消息的ID
        String messageId = message.getMessageProperties().getMessageId();
        try {
            //添加消息id到redis set集合中  添加成功返回1 表示未消费    添加失败返回0 表示已消费
            Long count = redisTemplate.opsForSet().add("messageId", messageId);
            //添加成功  正常消费信息
            if (count == 1) {
                log.info("开始消费");
                DruidDataSourceFactory druidDataSourceFactory = SpringUtils.getBean(DruidDataSourceFactory.class);
                DynamicDataSource dynamicDataSource = SpringUtils.getBean(DynamicDataSource.class);
                EntInfo entinfo = JSON.parseObject(msg, EntInfo.class);
                DataSourceInfo dataSourceInfo = DataSourceInfo.hostAndPortBuild(entinfo.getEntCode(), entinfo.getIp(), Math.toIntExact(entinfo.getPort()));
                DruidDataSource druidDataSource = druidDataSourceFactory.create(dataSourceInfo);
                dynamicDataSource.put(dataSourceInfo.getKey(), druidDataSource);
                //确认消费
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                log.info("消费成功");
            }
        } catch (Exception e) {
            //删除队列ID
            log.info("消费重复");
            try {
                //回退消息
                channel.basicReject(message.getMessageProperties().getDeliveryTag(), true);
                log.info("消费失败");
            } catch (IOException ex) {
                //回退失败
                log.info("消费异常");
            }
        }
    }

    private List<EnterpriseInfo> dataSourceInfoList() {
        Result<List<EntInfo>> entInfoResult = remoteEntInfoService.list();
        List<EntInfo> entInfoList = entInfoResult.getData();
        List<EnterpriseInfo> list = new ArrayList<>();
        entInfoList.forEach(entInfo -> {
            list.add(
                    EnterpriseInfo.builder()
                            .entCode(entInfo.getEntCode())
                            .ip(entInfo.getIp())
                            .port(String.valueOf(entInfo.getPort()))
                            .build()
            );
        });
        return list;
    }

    @Bean
    @Primary
    public DynamicDataSource dynamicDataSource(DruidDataSourceFactory druidDataSourceFactory) {

        Map<Object, Object> dataSourceMap = new HashMap<>();
        dataSourceInfoList()
                .stream()
                .map(enterpriseInfo -> DataSourceInfo.hostAndPortBuild(enterpriseInfo.getEntCode(), enterpriseInfo.getIp(), Integer.valueOf(enterpriseInfo.getPort())))
                .forEach(dataSourceInfo -> {
                    dataSourceMap.put(dataSourceInfo.getKey(), druidDataSourceFactory.create(dataSourceInfo));
                });
        //设置动态数据源
        DynamicDataSource dynamicDataSource = new DynamicDataSource();
//        dynamicDataSource.setDefaultTargetDataSource(masterDataSource());
        dynamicDataSource.setTargetDataSources(dataSourceMap);
        //将数据源信息备份在defineTargetDataSources中
        dynamicDataSource.setDefaultTargetDataSource(dataSourceMap);
        return dynamicDataSource;
    }

}
